/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
namespace IRDB_SDK
{

	using namespace std;

	class DecodedInstruction_t
	{
		protected:
			DecodedInstruction_t() {};
			DecodedInstruction_t(const DecodedInstruction_t& copy) = delete;
		public:

			virtual ~DecodedInstruction_t() {}

			virtual string   getDisassembly()               const = 0;
			virtual bool     valid()                        const = 0;
			virtual uint32_t length()                       const = 0;
			virtual bool     isBranch()                     const = 0;
			virtual bool     isCall()                       const = 0;
			virtual bool     isUnconditionalBranch()        const = 0;
			virtual bool     isConditionalBranch()          const = 0;
			virtual bool     isReturn()                     const = 0;
			virtual string   getMnemonic()                  const = 0;
			virtual int64_t  getImmediate()                 const = 0;
			virtual VirtualOffset_t        
			                 getAddress()                   const = 0;
			virtual bool     setsStackPointer()             const = 0;
			virtual uint32_t getPrefixCount()               const = 0;
			virtual bool     hasRelevantRepPrefix()         const = 0;
			virtual bool     hasRelevantRepnePrefix()       const = 0;
			virtual bool     hasRelevantOperandSizePrefix() const = 0;
			virtual bool     hasRexWPrefix()                const = 0;
			virtual bool     hasImplicitlyModifiedRegs()    const = 0;
			virtual bool     hasOperand(const int op_num)   const = 0;
			virtual shared_ptr<DecodedOperand_t>       
			                 getOperand(const int op_num)   const = 0;
			virtual DecodedOperandVector_t 
			                 getOperands()                  const = 0;
			virtual VirtualOffset_t        
			                 getMemoryDisplacementOffset(
			                                             const DecodedOperand_t* t, 
			                                             const Instruction_t* insn
			                                            )  const = 0;

			// factories
			static unique_ptr<DecodedInstruction_t> factory(const Instruction_t* p_in);
			static unique_ptr<DecodedInstruction_t> factory(const VirtualOffset_t start_addr, const void *data, uint32_t max_len);
			static unique_ptr<DecodedInstruction_t> factory(const VirtualOffset_t start_addr, const void *data, const void* endptr);


	};

	class DecodedOperand_t
	{
		protected:
			DecodedOperand_t() {}
			DecodedOperand_t(const DecodedOperand_t& copy) = delete;
		public:
			virtual ~DecodedOperand_t() {}

			virtual bool            isConstant()                        const = 0;
			virtual uint64_t        getConstant()                       const = 0;
			virtual string          getString()                         const = 0;
			virtual bool            isRegister()                        const = 0;
			virtual bool            isGeneralPurposeRegister()          const = 0;
			virtual bool            isMmxRegister()                     const = 0;
			virtual bool            isFpuRegister()                     const = 0;
			virtual bool            isSseRegister()                     const = 0;
			virtual bool            isAvxRegister()                     const = 0;
			virtual bool            isZmmRegister()                     const = 0;
			virtual bool            isSpecialRegister()                 const = 0;
			virtual bool            isSegmentRegister()                 const = 0;
			virtual uint32_t        getRegNumber()                      const = 0;
			virtual bool            isMemory()                          const = 0;
			virtual bool            hasSegmentRegister()                const = 0;
			virtual uint32_t        getSegmentRegister()                const = 0;
			virtual bool            hasBaseRegister()                   const = 0;
			virtual bool            hasIndexRegister()                  const = 0;
			virtual uint32_t        getBaseRegister()                   const = 0;
			virtual uint32_t        getIndexRegister()                  const = 0;
			virtual bool            hasMemoryDisplacement()             const = 0;
			virtual VirtualOffset_t getMemoryDisplacement()             const = 0;
			virtual bool            isPcrel()                           const = 0;
			virtual uint32_t        getScaleValue()                     const = 0;
			virtual uint32_t        getMemoryDisplacementEncodingSize() const = 0;
			virtual uint32_t        getArgumentSizeInBytes()            const = 0;
			virtual uint32_t        getArgumentSizeInBits()             const = 0;
			virtual bool            isRead()                            const = 0;
			virtual bool            isWritten()                         const = 0;

	};
}
