/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

namespace IRDB_SDK
{
	using namespace std;

	class Instruction_t : virtual public BaseObj_t
	{
		protected:
			Instruction_t() {}
			Instruction_t(const Instruction_t& copy) = delete;
		public:

			virtual AddressID_t*   getAddress()                     const = 0;
			virtual Function_t*    getFunction()                    const = 0;
			virtual DatabaseID_t   getOriginalAddressID()           const = 0;
			virtual Instruction_t* getFallthrough()                 const = 0;
			virtual Instruction_t* getTarget()                      const = 0;
			virtual ICFS_t*        getIBTargets()                   const = 0;
			virtual string         getDataBits()                    const = 0;
			virtual string         getCallback()                    const = 0;
			virtual string         getComment()                     const = 0;
			virtual EhProgram_t*   getEhProgram()                   const = 0;
			virtual EhCallSite_t*  getEhCallSite()                  const = 0;
			virtual AddressID_t*   getIndirectBranchTargetAddress() const = 0;
			virtual bool           isFunctionExit()                 const = 0;
			virtual string         getDisassembly()                 const = 0;
	  
			virtual void setAddress(AddressID_t* newaddr)                       = 0;
			virtual void setFunction(Function_t* func   )                       = 0;
			virtual void setOriginalAddressID(DatabaseID_t origid)              = 0;
			virtual void setFallthrough(Instruction_t* i)                       = 0;
			virtual void setTarget(Instruction_t* i)                            = 0;
			virtual void setIBTargets(ICFS_t *p_icfs)                           = 0;
			virtual void setDataBits(string orig)	                            = 0;
			virtual void setCallback(string orig)	                            = 0;
			virtual void setComment(string orig)	                            = 0;
			virtual void setEhProgram(EhProgram_t* orig)                        = 0;
			virtual void setEhCallSite(EhCallSite_t* orig)                      = 0;
			virtual void setIndirectBranchTargetAddress(AddressID_t* myIndTarg) = 0;
			virtual bool assemble(string assembly)                              = 0;

	};
}
