/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <string>

namespace IRDB_SDK
{
	using namespace std;

	using RegisterID_t = enum RegisterID
	{
		/* an register that's not valid on any machine, equiv. to a nullptr of registers */
		rn_UNKNOWN,

		/* x86 32 and 64   */
		rn_EFLAGS,

		/* x86 IP register names */
		rn_IP, rn_EIP=rn_IP, rn_RIP=rn_IP,

		/* x86 general purpose registers */
		rn_EAX, rn_EBX, rn_ECX, rn_EDX, rn_ESI, rn_EDI, rn_EBP, rn_ESP, rn_R8D, rn_R9D,  rn_R10D, rn_R11D, rn_R12D, rn_R13D, rn_R14D, rn_R15D,
		rn_RAX, rn_RBX, rn_RCX, rn_RDX, rn_RBP, rn_RSP, rn_RSI, rn_RDI, rn_R8,  rn_R9 ,  rn_R10,  rn_R11,  rn_R12,  rn_R13,  rn_R14,  rn_R15,
		rn_AX,  rn_BX,  rn_CX,  rn_DX,  rn_BP,  rn_SP,  rn_SI,  rn_DI,  rn_R8W, rn_R9W,  rn_R10W, rn_R11W, rn_R12W, rn_R13W, rn_R14W, rn_R15W,
		rn_AH,  rn_BH,  rn_CH,  rn_DH,  rn_SIH, rn_DIH, rn_BPH, rn_SPH,
		rn_AL,  rn_BL,  rn_CL,  rn_DL,  rn_SIL, rn_DIL, rn_BPL, rn_SPL, rn_R8B, rn_R9B,  rn_R10B, rn_R11B, rn_R12B, rn_R13B, rn_R14B, rn_R15B,

		/* other x86 registers here (e.g., fp, xmm, etc.), eventually */

		/* other archs support added here, eventually */
	};

	RegisterID_t strToRegister          (const string& p_regStr);
	RegisterID_t strToRegister          (const char  * p_regStr);
	bool         isValidRegister        (const RegisterID_t p_reg);
	bool         is64bitRegister        (const RegisterID_t p_reg);
	bool         is32bitRegister        (const RegisterID_t p_reg);
	bool         is16bitRegister        (const RegisterID_t p_reg);
	bool         is8bitRegister         (const RegisterID_t p_reg);
	int          getRegisterBitWidth    (const RegisterID_t p_reg);
	string       registerToString       (const RegisterID_t p_reg);
	RegisterID_t convertRegisterTo64bit (const RegisterID_t p_reg);
	RegisterID_t convertRegisterTo32bit (const RegisterID_t p_reg);
	RegisterID_t convertRegisterTo16bit (const RegisterID_t p_reg);
	RegisterID_t convertRegisterTo8bit  (const RegisterID_t p_reg);

}
